from .storage import Store as Store
from .typing import EventType as EventType, UNDEFINED as UNDEFINED, UndefinedType as UndefinedType
from _typeshed import Incomplete
from collections import UserDict
from collections.abc import Iterable, ValuesView
from dataclasses import dataclass
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.util import slugify as slugify
from typing import Literal, TypedDict

DATA_REGISTRY: str
EVENT_LABEL_REGISTRY_UPDATED: str
STORAGE_KEY: str
STORAGE_VERSION_MAJOR: int
SAVE_DELAY: int

class EventLabelRegistryUpdatedData(TypedDict):
    action: Literal['create', 'remove', 'update']
    label_id: str
EventLabelRegistryUpdated = EventType[EventLabelRegistryUpdatedData]

@dataclass(slots=True, frozen=True)
class LabelEntry:
    label_id: str
    name: str
    normalized_name: str
    description: str | None = ...
    color: str | None = ...
    icon: str | None = ...
    def __init__(self, label_id, name, normalized_name, description, color, icon) -> None: ...

class LabelRegistryItems(UserDict[str, LabelEntry]):
    _normalized_names: Incomplete
    def __init__(self) -> None: ...
    def values(self) -> ValuesView[LabelEntry]: ...
    def __setitem__(self, key: str, entry: LabelEntry) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def get_label_by_name(self, name: str) -> LabelEntry | None: ...

class LabelRegistry:
    labels: LabelRegistryItems
    _label_data: dict[str, LabelEntry]
    hass: Incomplete
    _store: Incomplete
    def __init__(self, hass: HomeAssistant) -> None: ...
    def async_get_label(self, label_id: str) -> LabelEntry | None: ...
    def async_get_label_by_name(self, name: str) -> LabelEntry | None: ...
    def async_list_labels(self) -> Iterable[LabelEntry]: ...
    def _generate_id(self, name: str) -> str: ...
    def async_create(self, name: str, *, color: str | None = None, icon: str | None = None, description: str | None = None) -> LabelEntry: ...
    def async_delete(self, label_id: str) -> None: ...
    def async_update(self, label_id: str, *, color: str | None | UndefinedType = ..., description: str | None | UndefinedType = ..., icon: str | None | UndefinedType = ..., name: str | UndefinedType = ...) -> LabelEntry: ...
    async def async_load(self) -> None: ...
    def async_schedule_save(self) -> None: ...
    def _data_to_save(self) -> dict[str, list[dict[str, str | None]]]: ...

def async_get(hass: HomeAssistant) -> LabelRegistry: ...
async def async_load(hass: HomeAssistant) -> None: ...
def _normalize_label_name(label_name: str) -> str: ...
